#!/usr/bin/python

import sys, getopt, os.path, shutil

VERSION = "0.1"

def main(argv):

	help = 'Usage: gamesetup.py -p <packagename> -i <internal project name> -d <project display name> -o <output directory>\n'

	try:
		opts, args = getopt.getopt(argv, "hvp:i:d:o:")
	except getopt.GetoptError:
		sys.stdout.write(help)
		sys.exit(2)

	for opt, arg in opts:
		if opt == '-h':
			sys.stdout.write(help)
			sys.exit()
		elif opt == '-v':
			print "LibGDX game setup script version", VERSION
			print "Copyright 2013 blue banana games"
			print "www.bluebanana-games.com"
			sys.exit()
		elif opt in ("-p", "--package"):
			packagename = arg.strip()
		elif opt in ("-i", "--internalname"):
			internalname = arg.strip()
		elif opt in ("-d", "--displayname"):
			displayname = arg.strip()
		elif opt in ("-o", "--outdir"):
			outdir = arg.strip()

   	sys.stdout.write('package:\t' + packagename + '\n')
   	sys.stdout.write('internal name:\t' + internalname + '\n')
   	sys.stdout.write('display name:\t' + displayname + '\n')
   	sys.stdout.write('output dir:\t' + outdir + '\n')

   	# TODO: pull newest version from git
   	ensureOutputDirExists(outdir)
   	basedir = copyProjectsFolder(outdir, internalname)
   	renameProjects(basedir, internalname)
   	renameSourceFolders(basedir, internalname, packagename)
   	cleanup(basedir)
   	replacePackageNameAndGameClass(basedir, internalname, packagename)
   	renameGameClass(basedir, internalname, packagename)
   	replaceDisplayName(basedir, displayname, internalname, packagename)

   	print "\nFinished successfully. Further steps:\n"
   	print "1) Use the LibGDX Setup UI to update to the latest version of LibGDX:"
   	print "   http://www.aurelienribon.com/blog/2012/04/new-app-libgdx-project-setup/"
   	print "2) Import the projects in Eclipse via File > Import... > Existing Projects into Workspace"
   	print "3) Make a great game!\n"

def ensureOutputDirExists(outdir):
	if os.path.exists(outdir):
		return

	if not raw_input('\nOutput directory does not exist. Create it? (Y/n):').upper() == 'Y':
		exit(3)

	sys.stdout.write('\n- creating "' + outdir + '"' + '\n')
	os.makedirs(outdir)


def copyProjectsFolder(outdir, internalname):
	destination = os.path.join(outdir, internalname)
	sys.stdout.write('- copying projects to "' + destination + '"' + '\n')
	shutil.copytree(os.path.dirname(os.path.realpath(__file__)), destination)
	return destination


def renameProjects(basedir, internalname):
	sys.stdout.write('- renaming project directories\n')
	for directory in os.listdir(basedir):
		if "GameTemplate" not in directory:
			continue

		src = os.path.join(basedir, directory)
		dst = os.path.join(basedir, directory.replace("GameTemplate", internalname))
		sys.stdout.write(src + " > " + dst + '\n')
		shutil.move(src, dst)

def renameSourceFolders(basedir, internalname, packagename):
	sys.stdout.write('- renaming source directories\n')

	packagePath = pathFromPackageName(packagename)

	for directory in os.listdir(basedir):
		projectdir = os.path.join(basedir, directory)
		if internalname not in directory:
			continue
		if not os.path.isdir(projectdir):
			continue

		newPackageDir = os.path.join(projectdir, "src", packagePath)
		oldPackageDir = os.path.join(projectdir, "src", pathFromPackageName("com.bluebananagames.gametemplate"))
		shutil.copytree(oldPackageDir, newPackageDir)
		shutil.rmtree(oldPackageDir)

def replacePackageNameAndGameClass(basedir, internalname, packagename):
	sys.stdout.write('- renaming package and main game class in java files\n')

	fileList = []

	for root, subFolders, files in os.walk(basedir):
		for fileName in files:
			if fileName.endswith(".java") or fileName == "AndroidManifest.xml" or fileName == ".classpath" or fileName == ".project":
				filePath = os.path.join(root, fileName)
				fileList.append(filePath)

	gameClassName = internalname + "Game"
	for fileName in fileList:
		print fileName

		contents = readFromFile(fileName)

		if fileName.endswith(".classpath") or fileName.endswith(".project"):		
			contents = contents.replace("GameTemplate", internalname)
		else:
			contents = contents.replace("com.bluebananagames.gametemplate", packagename)
			contents = contents.replace("GameTemplateGame", gameClassName)

		writeToFile(fileName, contents)

def renameGameClass(basedir, internalname, packagename):
	sys.stdout.write('- renaming game class\n')
	projectdir = os.path.join(basedir, internalname)
	packagePath = pathFromPackageName(packagename)
	oldFilePath = os.path.join(projectdir, "src", packagePath, "GameTemplateGame.java")
	newFilePath = os.path.join(projectdir, "src", packagePath, internalname + "Game.java")
	shutil.move(oldFilePath, newFilePath)

def replaceDisplayName(basedir, displayname, internalname, packagename):
	sys.stdout.write('- replacing display name\n')
	
	desktopProjectdir = os.path.join(basedir, internalname + "-desktop")
	packagePath = pathFromPackageName(packagename)
	mainFilePath = os.path.join(desktopProjectdir, "src", packagePath, "Main.java")

	contents = readFromFile(mainFilePath)
	contents = contents.replace("Game Template", displayname)
	writeToFile(mainFilePath, contents)
	
	androidProjectdir = os.path.join(basedir, internalname + "-android")
	stringsFilePath = os.path.join(androidProjectdir, "res", "values", "strings.xml")
	contents = readFromFile(stringsFilePath)
	contents = contents.replace("Game Template", displayname)
	writeToFile(stringsFilePath, contents)


def cleanup(basedir):
	# delete bin, gen and .git folders

	for root, subFolders, files in os.walk(basedir):
		for folder in subFolders:
			if folder == "bin" or folder == "gen" or folder == ".git":
				folderPath = os.path.join(root, folder)
				shutil.rmtree(folderPath)

	os.remove(os.path.join(basedir, "gamesetup.py"))


# HELPERS

def readFromFile(filePath):
	fileObject = open(filePath, 'r')
	contents = fileObject.read()
	fileObject.close()
	return contents

def writeToFile(filePath, contents):
	fileObject = open(filePath, 'w')
	fileObject.write(contents)
	fileObject.close()

def pathFromPackageName(packagename):
	packagePath = ''
	for component in packagename.split("."):
		packagePath = os.path.join(packagePath, component)

	return packagePath




if __name__ == "__main__":
   main(sys.argv[1:])

